import Bindings, Node

class Action(Node.Node):

    def __init__(self, name):
        Node.Node.__init__(self)
        self.name = name

    def __call__(self, binding):
        raise NotImplementedError()

    def bindings(self):
        return [ binding
                 for binding in self.owners
                 if isinstance(binding, Bindings.Binding) ]


class _ActionRegistry(object):

    def __init__(self):
        self._registry = {}

    def register(self, action):
        self._registry[action.name] = action

    def __getitem__(self, key):
        return self._registry[key]

Actions = _ActionRegistry()


class Nop(Action):

    def __init__(self, name='nop'):
        Action.__init__(self, name)

    def __call__(self, binding):
        pass
    

class Macro(Action):

    _actions = Node.NodeList()

    def __init__(self, name, actions):
        Action.__init__(self, name)
        for i in range(len(actions)):
            if type(actions[i]) is str:
                actions[i] = Actions[actions[i]]
        self._actions = actions

    def __call__(self, binding):
        for action in self._actions:
            action(binding)
            
class Command(Action):

    def __init__(self, name, callable):
        Action.__init__(self, name)
        self._callable = callable

    def __call__(self, binding):
        self._callable(binding)
        

class ChangeBindingsAbsolute(Action):

    def __init__(self, name, mapIndex, keymaps):
        Action.__init__(self, name)
        self._mapIndex = mapIndex
        self._keymaps = keymaps

    def __call__(self, binding):
        binding.keymap().keylist().replaceFromIndex(self._mapIndex, self._keymaps)


class ChangeBindingsRelative(Action):

    def __init__(self, name, mapIndex, keymaps):
        Action.__init__(self, name)
        self._mapIndex = mapIndex
        self._keymaps = keymaps

    def __call__(self, binding):
        keymap = binding.keymap()
        keymap.keylist().replaceFromIndex(keymap.index()+self._mapIndex, self._keymaps)

def action(func):
    action = Command(func.__name__, func)
    Actions.register(action)
    return func
