"""The Bindings module manages an infrastructure for key bindings."""

# Event = (context, sequence ...) wobei alle elsts strings sind

# gebuynbden wird ein Action object. Jede Action hat einen namen unter
# der sie registriert wird.  die funktionen zum definieren eines
# keymnap und zum binden von keys nehmen diesen namen als argument =>
# alle Aktionen m"ussen vorher registriert werden

# KeyStack => KeyMap => Binding => Action

# Das Binding Objekt enthaelt das Event unter welchem die Action
# gebunden ist. Jedes Binding aht auch einen label zum anzeigen des
# bindings

import Node, Actions, Events, Logger

class Binding(Node.Node):

    action = Node.NodeReference()

    def __init__(self, event, label, action):
        Node.Node.__init__(self)
        self.event = event
        self.label = label
        self.action = action

    def keymap(self):
        return self.owner

    def execute(self):
        Logger.log('binding', 'execute: %s' % self.action.name)
        self.action(self)


class KeyMap(Node.Node):

    _map = Node.NodeMap()

    def __init__(self, name=None):
        Node.Node.__init__(self)
        self._name = name

    def add(self, binding):
        if self.owner : raise RuntimeError, 'changing an active KeyMap not supported'
        self._map[binding.event] = binding

    def unbind(self, event):
        if self.owner : raise RuntimeError, 'changing an active KeyMap not supported'
        del self._map[event]

    def name(self):
        return self._name

    def index(self):
        if self.owner : return self.owner.indexOf(self)
        else          : return None

    def keylist(self):
        return self.owner

    def lookup(self, event):
        return self._map.get(event,None)

    def bindings(self):
        return self._map


class KeyList(Node.Node):

    _list = Node.NodeList()
    
    def __init__(self):
        Node.Node.__init__(self)
        self._callbacks = []

    def indexOf(self, map):
        return self._list.index(map)

    def append(self, map):
        self._list.append(map)
        self._callCallbacks()

    def prepend(self, map):
        self._list[:0] = [map]
        self._callCallbacks()

    def removeMap(self, map):
        for i in (range(len(self._list))):
            if self._list[i] is map:
                self._list[i:i+1] = []
                return

    def replaceFromIndex(self, index, maps):
        self._list[index:] = maps
        self._callCallbacks()

    def lookup(self, event):
        for i in range(len(self._list)-1,-1,-1):
            binding = self._list[i].lookup(event)
            if binding is not None : return binding
        return None

    def bindings(self):
        bindings = {}
        for map in self._list:
            bindings.update(map.bindings())
        return bindings

    def bindingsByContext(self):
        bindings = {}
        for map in self._list:
            for binding in map.bindings().itervalues():
                bindings.setdefault(binding.event.context,{})
                bindings[binding.event.context][binding.event] = binding
        return bindings

    def registerCallback(self, cb):
        self._callbacks.append(cb)

    def unregisterCallback(self, cb):
        self._callbacks.remove(cb)

    def _callCallbacks(self):
        for cb in self._callbacks : cb(self)

    def path(self):
        return ' / '.join([ map.name() for map in self._list if map.name() ])
