import Actions, OSC, Events

class Controller(object):

    def __init__(self, context, oscserver, remote):
        self._context = context
        self._server = oscserver
        self._remote = remote
        self._server.registerHandler(self._remote, self._oscEvent)

    def hit(self, command, index=0):
        self._server.command(self._remote,"/sl/%d/hit" % index, str(command))

    def get(self, parameter, index=0):
        if parameter.startswith('global_'):
            path = "/get"
            parameter = parameter[7:]
        else:
            path = "/sl/%d/get" % index
        self._server.command(self._remote, path, parameter, self._server.localUrl(), "")

    def set(self, parameter, value, index=0):
        if parameter.startswith('global_'):
            path = "/set"
            parameter = parameter[7:]
        else:
            path = "/sl/%d/set" % index
        self._server.command(self._remote, path, parameter, float(value))

    def _oscEvent(self, path, data):
        # data = (loop-number, parameter-name, value)
        if len(data) != 3: return
        if data[0]<0:
            param = "global_%s" % data[1]
        else:
            param = "%s/%d" % (data[1], data[0])
        return [ Events.ControlEvent(self._context, param, data[2]) ]

    class ParamGetter(object):

        def __init__(self, controller, parameter, index=0):
            self._parameter = parameter
            self._controller = controller
            self._index = index

        def __call__(self):
            self._controller.get(self._parameter, self._index)

    class ParamSetter(object):

        def __init__(self, controller, parameter, index=0):
            self._parameter = parameter
            self._controller = controller
            self._index = index

        def __call__(self, value):
            self._controller.set(self._parameter, value, self._index)

    def assignController( self, controller, title, parameter, min, max, stops=[], index=0 ):
        if parameter.startswith('global_'):
            event = parameter
        else:
            event = "%s/%d" % (parameter, index)
        controller.assign(title,
                          self.ParamSetter(self, parameter, index),
                          self.ParamGetter(self, parameter, index),
                          Events.Event(self._context, event),
                          min, max, stops)
                      
class Command(Actions.Action):

    def __init__(self, name, controller, command):
        Actions.Action.__init__(self, name)
        self._controller = controller
        self._command = command

    def __call__(self, binding):
        self._controller.hit(self._command)

class AssignController(Actions.Action):

    def __init__(self, name, loopController, pedalController, title, parameter, min, max,
                 stops = [], index=0):
        Actions.Action.__init__(self, name)
        self._loopController = loopController
        self._pedalController = pedalController
        self._title = title
        self._parameter = parameter
        self._min = min
        self._max = max
        self._stops = stops
        self._index = index

    def __call__(self, binding):
        self._loopController.assignController(self._pedalController,
                                              self._title,
                                              self._parameter,
                                              self._min,
                                              self._max,
                                              self._stops,
                                              self._index)

def register( oscserver,
              context,
              remote ):
    return Controller(context, oscserver, remote)
